@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2001 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2001-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -48,7 +48,7 @@ struct _GMimeHeader {
 static int
 header_equal (gconstpointer v, gconstpointer v2)
 {
-	return g_strcasecmp ((const char *) v, (const char *) v2) == 0;
+	return strcasecmp ((const char *) v, (const char *) v2) == 0;
 }
 
 static guint
@@ -335,7 +335,8 @@ g_mime_header_to_string (const GMimeHeader *header)
 	stream = g_mime_stream_mem_new ();
 	g_mime_stream_mem_set_byte_array (GMIME_STREAM_MEM (stream), array);
 	g_mime_header_write_to_stream (header, stream);
-	g_mime_stream_unref (stream);
+	g_object_unref (stream);
+	
 	g_byte_array_append (array, "", 1);
 	str = array->data;
 	g_byte_array_free (array, FALSE);
