@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2003 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2003-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -24,7 +24,14 @@
 #ifndef __GMIME_HOST_UTILS_H__
 #define __GMIME_HOST_UTILS_H__
 
+#include <glib.h>
+
+#include <sys/types.h>
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#else
 #include <netdb.h>
+#endif
 
 #ifdef __cplusplus
 extern "C" {
@@ -32,11 +39,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 int g_gethostbyname_r (const char *name, struct hostent *host,
-		       char *buf, size_t buflen, int *herr);
+		       char *buf, size_t buflen, GError **err);
 
 int g_gethostbyaddr_r (const char *addr, int addrlen, int af,
 		       struct hostent *host, char *buf,
-		       size_t buflen, int *herr);
+		       size_t buflen, GError **err);
 
 #ifdef __cplusplus
 }
