@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@
 #include "gmime-utils.h"
 
 
-#define d(x) x
+#define d(x)
 
 /* GObject class methods */
 static void g_mime_multipart_class_init (GMimeMultipartClass *klass);
@@ -144,7 +144,7 @@ g_mime_multipart_finalize (GObject *object)
 		GMimeObject *part;
 		
 		part = node->data;
-		g_mime_object_unref (part);
+		g_object_unref (part);
 		node = node->next;
 	}
 	g_list_free (multipart->subparts);
@@ -419,7 +419,7 @@ g_mime_multipart_get_postface (GMimeMultipart *multipart)
 static void
 multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 {
-	g_mime_object_ref (part);
+	g_object_ref (part);
 	multipart->subparts = g_list_append (multipart->subparts, part);
 }
 
@@ -444,7 +444,7 @@ g_mime_multipart_add_part (GMimeMultipart *multipart, GMimeObject *part)
 static void
 multipart_add_part_at (GMimeMultipart *multipart, GMimeObject *part, int index)
 {
-	g_mime_object_ref (part);
+	g_object_ref (part);
 	multipart->subparts = g_list_insert (multipart->subparts, part, index);
 }
 
@@ -500,7 +500,7 @@ multipart_remove_part (GMimeMultipart *multipart, GMimeObject *part)
 	
 	g_list_free_1 (node);
 	
-	g_mime_object_unref (part);
+	g_object_unref (part);
 }
 
 
@@ -582,7 +582,7 @@ multipart_get_part (GMimeMultipart *multipart, int index)
 	
 	part = node->data;
 	
-	g_mime_object_ref (part);
+	g_object_ref (part);
 	
 	return part;
 }
@@ -730,7 +730,7 @@ g_mime_multipart_get_boundary (GMimeMultipart *multipart)
  * @callback: function to call for @multipart and all of its subparts
  * @user_data: extra data to pass to the callback
  * 
- * Calls @callback on @multipart and each of its subparts.
+ * Calls @callback on each of @multipart's subparts.
  **/
 void
 g_mime_multipart_foreach (GMimeMultipart *multipart, GMimePartFunc callback, gpointer user_data)
