@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -41,6 +41,12 @@
 #define isblank(c) ((c) == ' ' || (c) == '\t')
 #endif /* HAVE_ISBLANK */
 
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
 #define d(x)
 
 static void g_mime_parser_class_init (GMimeParserClass *klass);
@@ -96,12 +102,13 @@ struct _GMimeParserPrivate {
 	off_t headers_start;
 	off_t header_start;
 	
-	unsigned int unstep:27;
+	unsigned int unstep:26;
 	unsigned int midline:1;
 	unsigned int seekable:1;
 	unsigned int scan_from:1;
 	unsigned int have_regex:1;
 	unsigned int persist_stream:1;
+	unsigned int respect_content_length:1;
 	
 	GMimeContentType *content_type;
 	struct _header_raw *headers;
@@ -112,9 +119,10 @@ struct _GMimeParserPrivate {
 struct _boundary_stack {
 	struct _boundary_stack *parent;
 	unsigned char *boundary;
-	unsigned int boundarylen;
-	unsigned int boundarylenfinal;
-	unsigned int boundarylenmax;
+	size_t boundarylen;
+	size_t boundarylenfinal;
+	size_t boundarylenmax;
+	off_t content_end;
 };
 
 static void
@@ -122,7 +130,7 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _boundary_stack *s;
-	unsigned int max;
+	size_t max;
 	
 	max = priv->bounds ? priv->bounds->boundarylenmax : 0;
 	
@@ -141,6 +149,8 @@ parser_push_boundary (GMimeParser *parser, const char *boundary)
 	}
 	
 	s->boundarylenmax = MAX (s->boundarylenfinal, max);
+	
+	s->content_end = -1;
 }
 
 static void
@@ -150,7 +160,7 @@ parser_pop_boundary (GMimeParser *parser)
 	struct _boundary_stack *s;
 	
 	if (!priv->bounds) {
-		g_warning ("boundary stack underflow");
+		d(g_warning ("boundary stack underflow"));
 		return;
 	}
 	
@@ -248,6 +258,7 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
 	parser->priv->persist_stream = TRUE;
+	parser->priv->respect_content_length = FALSE;
 }
 
 static void
@@ -273,7 +284,7 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	off_t offset = -1;
 	
 	if (stream) {
-		g_mime_stream_ref (stream);
+		g_object_ref (stream);
 		offset = g_mime_stream_tell (stream);
 	}
 	
@@ -312,7 +323,7 @@ parser_close (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	
 	if (priv->stream)
-		g_mime_stream_unref (priv->stream);
+		g_object_unref (priv->stream);
 	
 	g_byte_array_free (priv->from_line, TRUE);
 	
@@ -449,6 +460,24 @@ g_mime_parser_set_persist_stream (GMimeParser *parser, gboolean persist)
 }
 
 
+/**
+ * g_mime_parser_get_scan_from:
+ * @parser: MIME parser object
+ *
+ * Gets whether or not @parser is set to scan mbox-style From-lines.
+ *
+ * Returns whether or not @parser is set to scan mbox-style
+ * From-lines.
+ **/
+gboolean
+g_mime_parser_get_scan_from (GMimeParser *parser)
+{
+	g_return_val_if_fail (GMIME_IS_PARSER (parser), FALSE);
+	
+	return parser->priv->scan_from;
+}
+
+
 /**
  * g_mime_parser_set_scan_from:
  * @parser: MIME parser object
@@ -466,20 +495,42 @@ g_mime_parser_set_scan_from (GMimeParser *parser, gboolean scan_from)
 
 
 /**
- * g_mime_parser_get_scan_from:
+ * g_mime_parser_get_respect_content_length:
  * @parser: MIME parser object
  *
- * Gets whether or not @parser is set to scan mbox-style From-lines.
+ * Gets whether or not @parser is set to use Content-Length for
+ * determining the offset of the end of the message.
  *
- * Returns whether or not @parser is set to scan mbox-style
- * From-lines.
+ * Returns whether or not @parser is set to use Content-Length for
+ * determining the offset of the end of the message.
  **/
 gboolean
-g_mime_parser_get_scan_from (GMimeParser *parser)
+g_mime_parser_get_respect_content_length (GMimeParser *parser)
 {
 	g_return_val_if_fail (GMIME_IS_PARSER (parser), FALSE);
 	
-	return parser->priv->scan_from;
+	return parser->priv->respect_content_length;
+}
+
+
+/**
+ * g_mime_parser_set_respect_content_length:
+ * @parser: MIME parser object
+ * @respect_content_length: %TRUE if the parser should use Content-Length headers or %FALSE otherwise.
+ *
+ * Sets whether or not @parser should respect Content-Length headers
+ * when deciding where to look for the start of the next message. Only
+ * used when the parser is also set to scan for From-lines.
+ *
+ * Most notably useful when parsing broken Solaris mbox files (See
+ * http://www.jwz.org/doc/content-length.html for details).
+ **/
+void
+g_mime_parser_set_respect_content_length (GMimeParser *parser, gboolean respect_content_length)
+{
+	g_return_if_fail (GMIME_IS_PARSER (parser));
+	
+	parser->priv->respect_content_length = respect_content_length ? 1 : 0;
 }
 
 
@@ -561,8 +612,7 @@ parser_fill (GMimeParser *parser)
 	priv->inend = inbuf;
 	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF - 1;
 	
-	nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf);
-	if (nread > 0)
+	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0)
 		priv->inend += nread;
 	
 	priv->offset = g_mime_stream_tell (priv->stream);
@@ -572,11 +622,8 @@ parser_fill (GMimeParser *parser)
 
 
 static off_t
-parser_offset (GMimeParser *parser, unsigned char *cur)
+parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
-	struct _GMimeParserPrivate *priv = parser->priv;
-	unsigned char *inptr = cur;
-	
 	if (!inptr)
 		inptr = priv->inptr;
 	
@@ -599,7 +646,7 @@ g_mime_parser_tell (GMimeParser *parser)
 	g_return_val_if_fail (GMIME_IS_PARSER (parser), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (parser->priv->stream), -1);
 	
-	return parser_offset (parser, NULL);
+	return parser_offset (parser->priv, NULL);
 }
 
 
@@ -660,7 +707,7 @@ parser_step_from (GMimeParser *parser)
 			inptr++;
 			
 			if (len >= 5 && !strncmp (start, "From ", 5)) {
-				priv->from_offset = parser_offset (parser, start);
+				priv->from_offset = parser_offset (priv, start);
 				g_byte_array_append (priv->from_line, start, len);
 				goto got_from;
 			}
@@ -680,7 +727,7 @@ parser_step_from (GMimeParser *parser)
 
 #define header_backup(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
-		unsigned int hlen, hoff;                                  \
+		size_t hlen, hoff;                                        \
 		                                                          \
 		hlen = hoff = priv->headerptr - priv->headerbuf;          \
 		hlen = hlen ? hlen : 1;                                   \
@@ -701,7 +748,7 @@ parser_step_from (GMimeParser *parser)
 #define header_parse(parser, priv, hend) G_STMT_START {                   \
 	register unsigned char *colon;                                    \
 	struct _header_raw *header;                                       \
-	unsigned int hlen;                                                \
+	size_t hlen;                                                      \
 	                                                                  \
 	header = g_new (struct _header_raw, 1);                           \
 	header->next = NULL;                                              \
@@ -716,7 +763,7 @@ parser_step_from (GMimeParser *parser)
 	header->name = g_strndup (priv->headerbuf, hlen);                 \
 	g_strstrip (header->name);                                        \
 	if (*colon != ':') {                                              \
-		g_warning ("Invalid header: %s", header->name);           \
+		w(g_warning ("Invalid header: %s", header->name));        \
 		header->value = header->name;                             \
 		header->name = g_strdup ("X-Invalid-Header");             \
 	} else {                                                          \
@@ -748,8 +795,8 @@ parser_step_headers (GMimeParser *parser)
 	
 	priv->midline = FALSE;
 	hend = (struct _header_raw *) &priv->headers;
-	priv->headers_start = parser_offset (parser, NULL);
-	priv->header_start = parser_offset (parser, NULL);
+	priv->headers_start = parser_offset (priv, NULL);
+	priv->header_start = parser_offset (priv, NULL);
 	
 	inptr = priv->inptr;
 	inend = priv->inend;
@@ -772,7 +819,7 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
-			if (inptr + 1 >= inend) {
+			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
 				priv->inptr = start;
@@ -795,7 +842,7 @@ parser_step_headers (GMimeParser *parser)
 				} else {
 					priv->midline = FALSE;
 					header_parse (parser, priv, hend);
-					priv->header_start = parser_offset (parser, inptr);
+					priv->header_start = parser_offset (priv, inptr);
 				}
 			} else {
 				priv->midline = TRUE;
@@ -810,7 +857,6 @@ parser_step_headers (GMimeParser *parser)
 	inend = priv->inend;
 	
 	header_backup (priv, inptr, inend - inptr);
-	/*header_parse (priv, hend);*/
 	
  headers_end:
 	
@@ -832,8 +878,7 @@ parser_content_type (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	const char *content_type;
 	
-	content_type = header_raw_find (priv->headers, "Content-Type", NULL);
-	if (content_type)
+	if ((content_type = header_raw_find (priv->headers, "Content-Type", NULL)))
 		return g_mime_content_type_new_from_string (content_type);
 	
 	return NULL;
@@ -920,8 +965,10 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, int len)
+check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
 {
+	off_t offset = parser_offset (priv, (unsigned char *) start);
+	
 	if (possible_boundary (priv->scan_from, start, len)) {
 		struct _boundary_stack *s;
 		
@@ -930,10 +977,12 @@ check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, in
 		s = priv->bounds;
 		while (s) {
 			/* we use >= here because From lines are > 5 chars */
-			if (len >= s->boundarylenfinal &&
+			if (offset >= s->content_end &&
+			    len >= s->boundarylenfinal &&
 			    !strncmp (s->boundary, start,
 				      s->boundarylenfinal)) {
-				d(printf ("found end boundary\n"));
+				d(printf ("found %s\n", s->content_end != -1 && offset >= s->content_end ?
+					  "end of content" : "end boundary"));
 				return FOUND_END_BOUNDARY;
 			}
 			
@@ -1053,7 +1102,7 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	off_t start, end;
 	
 	if (priv->persist_stream && priv->seekable)
-		start = parser_offset (parser, NULL);
+		start = parser_offset (priv, NULL);
 	else
 		content = g_byte_array_new ();
 	
@@ -1061,11 +1110,11 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
-			end = parser_offset (parser, NULL) - 1;
+			end = parser_offset (priv, NULL) - 1;
 		else
 			g_byte_array_set_size (content, MAX (content->len - 1, 0));
 	} else if (priv->persist_stream && priv->seekable) {
-		end = parser_offset (parser, NULL);
+		end = parser_offset (priv, NULL);
 	}
 	
 	encoding = g_mime_part_get_encoding (mime_part);
@@ -1078,8 +1127,8 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	
 	wrapper = g_mime_data_wrapper_new_with_stream (stream, encoding);
 	g_mime_part_set_content_object (mime_part, wrapper);
-	g_mime_stream_unref (stream);
 	g_object_unref (wrapper);
+	g_object_unref (stream);
 }
 
 static void
@@ -1099,7 +1148,7 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (GMIME_OBJECT (message), header->name, header->value);
+		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1115,10 +1164,10 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	}
 	
 	g_mime_message_set_mime_part (message, object);
-	g_mime_object_unref (object);
+	g_object_unref (object);
 	
 	g_mime_message_part_set_message (mpart, message);
-	g_mime_object_unref (GMIME_OBJECT (message));
+	g_object_unref (message);
 }
 
 static GMimeObject *
@@ -1217,7 +1266,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		}
 		
 		g_mime_multipart_add_part (multipart, subpart);
-		g_mime_object_unref (subpart);
+		g_object_unref (subpart);
 	} while (found == FOUND_BOUNDARY);
 	
 	return found;
@@ -1272,12 +1321,13 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv)) {
 			/* eat end boundary */
 			parser_skip_line (parser);
+			parser_pop_boundary (parser);
 			*found = parser_scan_multipart_postface (parser, multipart);
+		} else {
+			parser_pop_boundary (parser);
 		}
-		
-		parser_pop_boundary (parser);
 	} else {
-		g_warning ("multipart without boundary encountered");
+		w(g_warning ("multipart without boundary encountered"));
 		/* this will scan everything into the preface */
 		*found = parser_scan_multipart_preface (parser, multipart);
 	}
@@ -1334,6 +1384,7 @@ parser_construct_message (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	GMimeContentType *content_type;
 	struct _header_raw *header;
+	int content_length = -1;
 	GMimeMessage *message;
 	GMimeObject *object;
 	int found;
@@ -1345,12 +1396,18 @@ parser_construct_message (GMimeParser *parser)
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (GMIME_OBJECT (message), header->name, header->value);
+		if (priv->respect_content_length && !strcasecmp (header->name, "Content-Length"))
+			content_length = strtoul (header->value, NULL, 10);
+		
+		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
-	if (priv->scan_from)
+	if (priv->scan_from) {
 		parser_push_boundary (parser, "From ");
+		if (priv->respect_content_length && content_length != -1)
+			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
+	}
 	
 	content_type = parser_content_type (parser);
 	if (!content_type)
@@ -1364,7 +1421,7 @@ parser_construct_message (GMimeParser *parser)
 	}
 	
 	g_mime_message_set_mime_part (message, object);
-	g_mime_object_unref (object);
+	g_object_unref (object);
 	
 	if (priv->scan_from) {
 		priv->state = GMIME_PARSER_STATE_FROM;
