@@ -42,6 +42,8 @@
 #include "gmime-filter-crlf.h"
 #include "gmime-filter-md5.h"
 
+#define d(x)
+
 /* GObject class methods */
 static void g_mime_part_class_init (GMimePartClass *klass);
 static void g_mime_part_init (GMimePart *mime_part, GMimePartClass *klass);
@@ -1047,6 +1049,9 @@ g_mime_part_get_filename (const GMimePart *mime_part)
  * @content: raw mime part content
  * @len: raw content length
  *
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_part_set_content_object instead.
+ *
  * Sets the content of the Mime Part (only non-multiparts)
  **/
 void
@@ -1071,6 +1076,9 @@ g_mime_part_set_content (GMimePart *mime_part, const char *content, size_t len)
  * @mime_part: Mime part
  * @content: raw mime part content.
  *
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_part_set_content_object instead.
+ *
  * Sets the content of the Mime Part (only non-multiparts)
  **/
 void
@@ -1097,7 +1105,11 @@ g_mime_part_set_content_byte_array (GMimePart *mime_part, GByteArray *content)
  * @len: length of the content
  * @encoding: content encoding
  *
- * Sets the encoding type and raw content on the mime part after decoding the content.
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_part_set_content_object instead.
+ *
+ * Sets the encoding type and raw content on the mime part after
+ * decoding the content.
  **/
 void
 g_mime_part_set_pre_encoded_content (GMimePart *mime_part, const char *content,
@@ -1197,10 +1209,16 @@ g_mime_part_get_content_object (const GMimePart *mime_part)
  *
  * Gets the raw contents of the mime part and sets @len to the length
  * of the raw data buffer.
+ *
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_get_content_object instead.
  * 
  * Returns a const char * pointer to the raw contents of the MIME Part
- * and sets %len to the length of the buffer. Note: textual content
- * will not be converted to UTF-8.
+ * and sets @len to the length of the buffer. Note: textual content
+ * will not be converted to UTF-8. Also note that this buffer will not
+ * be nul-terminated and may in fact contain nul bytes mid-buffer so
+ * you MUST treat the data returned as raw binary data even if the
+ * content type is text.
  **/
 const char *
 g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
@@ -1211,7 +1229,7 @@ g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
 	
 	if (!mime_part->content || !mime_part->content->stream) {
-		g_warning ("no content set on this mime part");
+		d(g_warning ("no content set on this mime part"));
 		return NULL;
 	}
 	
@@ -1260,6 +1278,9 @@ g_mime_part_get_content (const GMimePart *mime_part, size_t *len)
  *
  * Writes the contents of the MIME Part to @stream.
  *
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_object_write_to_stream instead.
+ *
  * Returns the number of bytes written or -1 on fail.
  **/
 ssize_t
@@ -1268,7 +1289,7 @@ g_mime_part_write_to_stream (GMimePart *mime_part, GMimeStream *stream)
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), -1);
 	g_return_val_if_fail (GMIME_IS_STREAM (stream), -1);
 	
-	return g_mime_object_write_to_stream (GMIME_OBJECT (mime_part), stream);
+	return g_mime_object_write_to_stream ((GMimeObject *) mime_part, stream);
 }
 
 
@@ -1278,6 +1299,9 @@ g_mime_part_write_to_stream (GMimePart *mime_part, GMimeStream *stream)
  *
  * Allocates a string buffer containing the MIME Part.
  *
+ * WARNING: This interface is deprecated. Use
+ * #g_mime_object_to_string instead.
+ *
  * Returns an allocated string containing the MIME Part.
  **/
 char *
@@ -1285,5 +1309,5 @@ g_mime_part_to_string (GMimePart *mime_part)
 {
 	g_return_val_if_fail (GMIME_IS_PART (mime_part), NULL);
 	
-	return g_mime_object_to_string (GMIME_OBJECT (mime_part));
+	return g_mime_object_to_string ((GMimeObject *) mime_part);
 }
