@@ -464,6 +464,32 @@ get_tzone (struct _date_token **token)
 	return -1;
 }
 
+static time_t
+mktime_utc (struct tm *tm)
+{
+	time_t tt;
+	
+	tm->tm_isdst = -1;
+	tt = mktime (tm);
+	
+#if defined (HAVE_TM_GMTOFF)
+	tt += tm->tm_gmtoff;
+#elif defined (HAVE_TIMEZONE)
+	if (tm->tm_isdst > 0) {
+#if defined (HAVE_ALTZONE)
+		tt -= altzone;
+#else /* !defined (HAVE_ALTZONE) */
+		tt -= (timezone - 3600);
+#endif
+	} else
+		tt -= timezone;
+#else
+#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
+#endif
+	
+	return tt;
+}
+
 static time_t
 parse_rfc822_date (struct _date_token *tokens, int *tzone)
 {
@@ -522,22 +548,7 @@ parse_rfc822_date (struct _date_token *tokens, int *tzone)
 		offset = n;
 	}
 	
-	t = mktime (&tm);
-#if defined (HAVE_TM_GMTOFF)
-	t += tm.tm_gmtoff;
-#elif defined (HAVE_TIMEZONE)
-	if (tm.tm_isdst > 0) {
-#if defined (HAVE_ALTZONE)
-		t -= altzone;
-#else /* !defined (HAVE_ALTZONE) */
-		t -= (timezone - 3600);
-#endif
-	} else {
-		t -= timezone;
-	}
-#else
-#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
-#endif
+	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
@@ -650,22 +661,7 @@ parse_broken_date (struct _date_token *tokens, int *tzone)
 	
 	d(printf ("\n"));
 		
-	t = mktime (&tm);
-#if defined (HAVE_TM_GMTOFF)
-	t += tm.tm_gmtoff;
-#elif defined (HAVE_TIMEZONE)
-	if (tm.tm_isdst > 0) {
-#if defined (HAVE_ALTZONE)
-		t -= altzone;
-#else /* !defined (HAVE_ALTZONE) */
-		t -= (timezone - 3600);
-#endif
-	} else {
-		t -= timezone;
-	}
-#else
-#error Neither HAVE_TIMEZONE nor HAVE_TM_GMTOFF defined. Rerun autoheader, autoconf, etc.
-#endif
+	t = mktime_utc (&tm);
 	
 	/* t is now GMT of the time we want, but not offset by the timezone ... */
 	
@@ -838,23 +834,30 @@ decode_msgid (const char **in)
 	char *msgid = NULL;
 	
 	decode_lwsp (&inptr);
-	if (*inptr == '<') {
-		inptr++;
-		decode_lwsp (&inptr);
-		if ((msgid = decode_addrspec (&inptr))) {
-			decode_lwsp (&inptr);
-			if (*inptr != '>') {
-				w(g_warning ("Invalid msg-id; missing '>': %s", *in));
-			} else {
-				inptr++;
-			}
-			
-			*in = inptr;
-		} else {
-			w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
-		}
-	} else {
+	if (*inptr != '<') {
 		w(g_warning ("Invalid msg-id; missing '<': %s", *in));
+	} else {
+		inptr++;
+	}
+	
+	decode_lwsp (&inptr);
+	if ((msgid = decode_addrspec (&inptr))) {
+		decode_lwsp (&inptr);
+		if (*inptr != '>') {
+			w(g_warning ("Invalid msg-id; missing '>': %s", *in));
+		} else {
+			inptr++;
+		}
+		
+		*in = inptr;
+	} else {
+		w(g_warning ("Invalid msg-id; missing addr-spec: %s", *in));
+		*in = inptr;
+		while (*inptr && *inptr != '>')
+			inptr++;
+		
+		msgid = g_strndup (*in, inptr - *in);
+		*in = inptr;
 	}
 	
 	return msgid;
@@ -997,19 +1000,19 @@ g_mime_utils_header_fold (const char *in)
 	
 	inptr = in;
 	len = strlen (in);
-	if (len <= GMIME_FOLD_LEN)
+	if (len <= GMIME_FOLD_LEN + 1)
 		return g_strdup (in);
 	
 	out = g_string_new ("");
 	outlen = 0;
-	while (*inptr) {
-		len = strcspn (inptr, " \t");
+	while (*inptr && *inptr != '\n') {
+		len = strcspn (inptr, " \t\n");
 		
-		if (outlen + len > GMIME_FOLD_LEN) {			
+		if (outlen + len > GMIME_FOLD_LEN) {
 			if (last_was_lwsp)
-				g_string_truncate (out, out->len - 1);
-			
-			g_string_append (out, "\n\t");
+				g_string_insert_c (out, out->len - 1, '\n');
+			else
+				g_string_append (out, "\n\t");
 			outlen = 1;
 			
 			/* check for very long words, just cut them up */
@@ -1030,8 +1033,8 @@ g_mime_utils_header_fold (const char *in)
 		} else {
 			if (*inptr == '\t') {
 				/* tabs are a good place to fold, odds
-                                   are that this is where the previous
-                                   mailer folded it */
+				   are that this is where the previous
+				   mailer folded it */
 				g_string_append (out, "\n\t");
 				outlen = 1;
 				inptr++;
@@ -1044,6 +1047,9 @@ g_mime_utils_header_fold (const char *in)
 		}
 	}
 	
+	if (*inptr == '\n' && out->str[out->len - 1] != '\n')
+		g_string_append_c (out, '\n');
+	
 	ret = out->str;
 	g_string_free (out, FALSE);
 	
@@ -1874,16 +1880,14 @@ g_string_append_len_quoted (GString *out, const char *in, size_t len)
 }
 
 static char *
-rfc2047_encode (const unsigned char *in, gboolean phrase)
+rfc2047_encode (const unsigned char *in, gushort safemask)
 {
 	struct _rfc822_word *words, *word, *prev = NULL;
-	gushort safemask = phrase ? IS_PSAFE : IS_ESAFE;
 	GString *out;
 	char *outstr;
 	
-	words = rfc2047_encode_get_rfc822_words (in, phrase);
-	if (!words)
-		return NULL;
+	if (!(words = rfc2047_encode_get_rfc822_words (in, safemask & IS_PSAFE)))
+		return g_strdup (in);
 	
 	while (rfc2047_encode_merge_rfc822_words (&words))
 		;
@@ -1908,7 +1912,7 @@ rfc2047_encode (const unsigned char *in, gboolean phrase)
 			g_string_append_len (out, word->start, word->end - word->start);
 			break;
 		case WORD_QSTRING:
-			g_assert (phrase);
+			g_assert (safemask & IS_PSAFE);
 			g_string_append_len_quoted (out, word->start, word->end - word->start);
 			break;
 		case WORD_2047:
@@ -1961,7 +1965,7 @@ g_mime_utils_header_encode_phrase (const unsigned char *in)
 	if (in == NULL)
 		return NULL;
 	
-	return rfc2047_encode (in, TRUE);
+	return rfc2047_encode (in, IS_PSAFE);
 }
 
 
@@ -1980,7 +1984,7 @@ g_mime_utils_header_encode_text (const unsigned char *in)
 	if (in == NULL)
 		return NULL;
 	
-	return rfc2047_encode (in, FALSE);
+	return rfc2047_encode (in, IS_ESAFE);
 }
 
 
@@ -2409,7 +2413,7 @@ g_mime_utils_uuencode_step (const unsigned char *in, size_t inlen, unsigned char
  * @save: leftover bits that have not yet been decoded
  *
  * Uudecodes a chunk of data. Performs a 'decode step' on a chunk of
- * uuencoded data. Assumes the "begin <mode> <file name>" line has
+ * uuencoded data. Assumes the "begin mode filename" line has
  * been stripped off.
  *
  * Returns the number of bytes decoded.
